/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of SRAM demo for the 
 *          TOSHIBA 'TMPM364' Device Series 
 * @version V0.141
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#include <string.h>
#include "tmpm364_smc.h"

uint8_t SMC_GetBusMode(void);

uint8_t SMC_GetBusMode(void)
{
    uint8_t BusMode = 0xFFU;

    BusMode = (uint8_t) TSB_SMCMD->MODE;
    return BusMode;
}

/* The demo is for external 16-bit SRAM, separate bus mode */
int main(void)
{
    uint8_t chip = 0xFFU;
    SMC_IFConfigTypeDef config = { 0U };
    uint8_t bus_w = 0xFFU;
    uint8_t bus_r = 0xFFU;
    SMC_CyclesTypeDef cycles_w = { 0U };
    SMC_CyclesTypeDef cycles_r = { 0U };
    SMC_OpModeTypeDef opmode_w = { 0U };
    SMC_OpModeTypeDef opmode_r = { 0U };
    SMC_DirectCMDTypeDef cmd = { 0U };
    uint8_t retval = 0U;

    /* Initial process demo */

    /* Herein describe SRAM model number and basic infomation */
    /* CS0: 0x60000000 ~ 0x60FFFFFF(16MB) */
    /* CS1: 0x61000000 ~ 0x61FFFFFF(16MB) */
    /* CS2: 0x62000000 ~ 0x62FFFFFF(16MB) */
    /* CS3: 0x63000000 ~ 0x63FFFFFF(16MB) */
    chip = SMC_CS2;

    bus_w = SMC_BUS_MULTIPLEX;  /* Address bus and data bus are multiplexed */
    SMC_SetBusMode(bus_w);

    bus_r = SMC_GetBusMode();
    if (bus_w != bus_r) {
        retval |= 0x01U;
    }

    SMC_GetIFConfig(&config);
    if (config.MemoryType != SMC_MULTIPLEX_SRAM) {
        /* If multiplex bus mode, MemoryType is SMC_MULTIPLEX_SRAM */
        retval = 0x02U;
    }
    if (config.MemoryWidth != SMC_DATA_BUS_16BIT) {
        /* SMC_DATA_BUS_16BIT can not be read out if no external memory, */
        /* and default value of "0x02" will be read out. */
        retval |= 0x04U;
    }
    if ((config.MemoryChips) != SMC_MEMORY_CHIPS_4) {
        retval |= 0x08U;
    }

    /* Set cycles time according to AC timing of SRAM datasheet,base clock: SMCCLK(fsys*1/2) */

    /* Separate bus mode:tCEOE + SMCCLK*1clk <= tRC  */
    /* Multiplex bus mode:tCEOE + SMCCLK*2clk <= tRC */
    cycles_w.RC_Time = SMC_READ_CYCLE_TIME_5;
    /* Separate bus mode:tWP + SMCCLK*2clk <= tWC  */
    /* Multiplex bus mode:tWP + SMCCLK*3clk <= tWC */
    cycles_w.WC_Time = SMC_WRITE_CYCLE_TIME_5;
    cycles_w.CEOE_Time = SMC_CEOE_DELAY_CYCLE_TIME_1;
    cycles_w.WP_Time = SMC_WE_PULSE_CYCLE_TIME_1;

    /* PC_Time is only effective for separate bus mode */
    /* It's necessary to also set to pass parameter check in multiplex bus mode */
    cycles_w.PC_Time = SMC_PAGE_CYCLE_TIME_5;

    cycles_w.TR_Time = SMC_TURN_AROUND_CYCLE_TIME_1;
    SMC_SetCycles(&cycles_w);

    opmode_w.BusWidth = SMC_DATA_BUS_16BIT;
    opmode_w.ReadBurstLen = SMC_READ_BURST_4BEAT;
    opmode_w.ALE = ENABLE;      /* ALE must be enabled in multiplex bus mode */
    SMC_SetOpMode(&opmode_w);

    /* Make sure SMC_SetCycles and SMC_SetOpMode are performed before SMC_SendDirectCMD */
    cmd.CmdType = SMC_CMD_UPDATEREGS;
    cmd.ChipSelect = chip;
    SMC_SendDirectCMD(&cmd);

    /* Read cycles and opmode for verify */
    SMC_GetCycles(chip, &cycles_r);
    if (memcmp(&cycles_r, &cycles_w, sizeof(SMC_CyclesTypeDef)) != 0) {
        retval |= 0x10U;
    }

    SMC_GetOpMode(chip, &opmode_r);
    opmode_w.StartAddr = SMC_START_ADDR_CHIP0 | (uint8_t) chip; /* For compare with read out opmode */
    if (memcmp(&opmode_r, &opmode_w, sizeof(SMC_OpModeTypeDef)) != 0) {
        retval |= 0x20U;
    }

    /* check retval to see if SRAM initialization is sucessful or not */
    if (retval == 0U) {
        /* Sucessful */
    } else {
        /* Failed */
    }

    while (1) {
        /* Do nothing */
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
